At the moment, each exposed public IP has its own firewall

<!-- START doctoc generated TOC please keep comment here to allow auto update -->
<!-- DON'T EDIT THIS SECTION, INSTEAD RE-RUN doctoc TO UPDATE -->
**Table of Contents**

- [Commands to operate](#commands-to-operate)
- [Details](#details)
  - [Clean iptables-save output](#clean-iptables-save-output)
  - [Testing new firewall rules](#testing-new-firewall-rules)

<!-- END doctoc generated TOC please keep comment here to allow auto update -->

# Commands to operate

save config current config: `iptables-save > /etc/firewall`

load saved config: `iptables-restore < /etc/firewall`

read saved config at boot (only one time):

```
cat > /etc/network/if-pre-up.d/firewall <<EOF
#!/bin/sh
/sbin/iptables-restore < /etc/firewall
EOF
chmod +x /etc/network/if-pre-up.d/firewall
```

Check applied rules: `iptables -L -v`

# Details

You have different ways to proceed:

put different `iptables` commands and save them, for example (source from early version of rproxy VM) you put these commands:

```
iptables -A INPUT -p icmp -m comment --comment "Allow ICMP" -j ACCEPT
iptables -A INPUT ! -i eth0 -m state --state ESTABLISHED,RELATED -m comment --comment "Allow outgoing connections responses, basically to permit software updates, NTP and other external services" -j ACCEPT
iptables -A INPUT -i lo -m comment --comment "Allow internal services to communicate" -j ACCEPT
iptables -A INPUT -i eth0 -j ACCEPT
iptables -A INPUT -i eth1 -p tcp -m multiport --dports 80,443 -j ACCEPT
iptables -A INPUT -i eth2 -p tcp -m multiport --dports 80,443 -j ACCEPT
iptables -A INPUT -j DROP
```

then, `iptables -L -v` says:

```
Chain INPUT (policy ACCEPT 0 packets, 0 bytes)
 pkts bytes target     prot opt in     out     source               destination         
   15  1024 ACCEPT     icmp --  any    any     anywhere             anywhere             /* Allow ICMP */
 1299  246K ACCEPT     all  --  !eth0  any     anywhere             anywhere             state RELATED,ESTABLISHED /* Allow outgoing connections responses, basically to permit software updates, NTP and other external services */
    0     0 ACCEPT     all  --  lo     any     anywhere             anywhere             /* Allow internal services to communicate */
 5023  857K ACCEPT     all  --  eth0   any     anywhere             anywhere            
    5   300 ACCEPT     tcp  --  eth1   any     anywhere             anywhere             multiport dports http,https
    0     0 ACCEPT     tcp  --  eth2   any     anywhere             anywhere             multiport dports http,https
  157 20990 DROP       all  --  any    any     anywhere             anywhere            

Chain FORWARD (policy ACCEPT 0 packets, 0 bytes)
 pkts bytes target     prot opt in     out     source               destination         

Chain OUTPUT (policy ACCEPT 6139 packets, 1019K bytes)
 pkts bytes target     prot opt in     out     source               destination  
```

if you use `iptables-save` you will have something like:

```
# Generated by iptables-save v1.4.21 on Wed Apr  5 10:20:18 2017
*mangle
:PREROUTING ACCEPT [8285:1433283]
:INPUT ACCEPT [8123:1421599]
:FORWARD ACCEPT [0:0]
:OUTPUT ACCEPT [7684:1304804]
:POSTROUTING ACCEPT [7684:1304804]
COMMIT
# Completed on Wed Apr  5 10:20:18 2017
# Generated by iptables-save v1.4.21 on Wed Apr  5 10:20:18 2017
*filter
:INPUT ACCEPT [0:0]
:FORWARD ACCEPT [0:0]
:OUTPUT ACCEPT [6792:1088067]
-A INPUT -p icmp -m comment --comment "Allow ICMP" -j ACCEPT
-A INPUT ! -i eth0 -m state --state RELATED,ESTABLISHED -m comment --comment "Allow outgoing connections responses, basically to permit software updates, NTP and other external services" -j ACCEPT
-A INPUT -i lo -m comment --comment "Allow internal services to communicate" -j ACCEPT
-A INPUT -i eth0 -j ACCEPT
-A INPUT -i eth1 -p tcp -m multiport --dports 80,443 -j ACCEPT
-A INPUT -i eth2 -p tcp -m multiport --dports 80,443 -j ACCEPT
-A INPUT -j DROP
COMMIT
# Completed on Wed Apr  5 10:20:18 2017
# Generated by iptables-save v1.4.21 on Wed Apr  5 10:20:18 2017
*nat
:PREROUTING ACCEPT [366:32656]
:INPUT ACCEPT [43:2735]
:OUTPUT ACCEPT [752:45144]
:POSTROUTING ACCEPT [752:45144]
COMMIT
# Completed on Wed Apr  5 10:20:18 2017
```

This is what we are going to put in a file to make iptables persistent

## Clean iptables-save output

Clean output of `iptables-save > /etc/firewall` without loosing functionality apply:

```
sed -i '/^#/d' /etc/firewall
sed -i 's/\[.*\]//g' /etc/firewall
```

The result is:

```
*mangle
:PREROUTING ACCEPT
:INPUT ACCEPT
:FORWARD ACCEPT
:OUTPUT ACCEPT
:POSTROUTING ACCEPT
COMMIT
*filter
:INPUT ACCEPT
:FORWARD ACCEPT
:OUTPUT ACCEPT
-A INPUT -p icmp -m comment --comment "Allow ICMP" -j ACCEPT
-A INPUT ! -i eth0 -m state --state RELATED,ESTABLISHED -m comment --comment "Allow outgoing connections responses, basically to permit software updates, NTP and other external services" -j ACCEPT
-A INPUT -i lo -m comment --comment "Allow internal services to communicate" -j ACCEPT
-A INPUT -i eth0 -j ACCEPT
-A INPUT -i eth1 -p tcp -m multiport --dports 80,443 -j ACCEPT
-A INPUT -i eth2 -p tcp -m multiport --dports 80,443 -j ACCEPT
-A INPUT -j DROP
COMMIT
*nat
:PREROUTING ACCEPT
:INPUT ACCEPT
:OUTPUT ACCEPT
:POSTROUTING ACCEPT
COMMIT
```

## Testing new firewall rules

Different firewall files can be saved to try different features, let's work with `firewall.test` file: `cp /etc/firewall /etc/firewall.test`

To do firewall modifications reload `/etc/firewall.test` commenting the parts you don't want. Let's bypass everything

```
*mangle
:PREROUTING ACCEPT
:INPUT ACCEPT
:FORWARD ACCEPT
:OUTPUT ACCEPT
:POSTROUTING ACCEPT
COMMIT
*filter
:INPUT ACCEPT
:FORWARD ACCEPT
:OUTPUT ACCEPT
# -A INPUT -p icmp -m comment --comment "Allow ICMP" -j ACCEPT
# -A INPUT ! -i eth0 -m state --state RELATED,ESTABLISHED -m comment --comment "Allow outgoing connections responses, basically to permit software updates, NTP and other external services" -j ACCEPT
# -A INPUT -i lo -m comment --comment "Allow internal services to communicate" -j ACCEPT
# -A INPUT -i eth0 -j ACCEPT
# -A INPUT -i eth1 -p tcp -m multiport --dports 80,443 -j ACCEPT
# -A INPUT -i eth2 -p tcp -m multiport --dports 80,443 -j ACCEPT
# -A INPUT -j DROP
COMMIT
*nat
:PREROUTING ACCEPT
:INPUT ACCEPT
:OUTPUT ACCEPT
:POSTROUTING ACCEPT
COMMIT
```

apply `iptables-restore < /etc/firewall.test`

then `iptables -L -v` says:

```
Chain INPUT (policy ACCEPT 80 packets, 16701 bytes)
 pkts bytes target     prot opt in     out     source               destination         

Chain FORWARD (policy ACCEPT 0 packets, 0 bytes)
 pkts bytes target     prot opt in     out     source               destination         

Chain OUTPUT (policy ACCEPT 65 packets, 15647 bytes)
 pkts bytes target     prot opt in     out     source               destination 
```

with `iptables-restore < /etc/firewall` original firewall is restored
